#  tests for anaconda-navigator-2.5.2-py311haa95532_0 (this is a generated file);
print('===== testing package: anaconda-navigator-2.5.2-py311haa95532_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import anaconda_navigator

print('anaconda_navigator.__version__: %s' % anaconda_navigator.__version__)
assert anaconda_navigator.__version__ == '2.5.2'
#  --- run_test.py (end) ---

print('===== anaconda-navigator-2.5.2-py311haa95532_0 OK =====');
print("import: 'anaconda_navigator'")
import anaconda_navigator

